@@ -113,7 +113,7 @@ public:
     TestCodeCompletionContext(KDevelop::DUContextPointer context, const QString& text, const QString& followingText, const CursorInRevision &position, int depth = 0)
         : CodeCompletionContext(context, text, followingText, position, depth) { }
 protected:
-    QList<QSet<IndexedString> > completionFiles() {
+    QList<QSet<IndexedString> > completionFiles() override {
         QList<QSet<IndexedString> > ret;
         QSet<IndexedString> set;
         set << IndexedString("file:///internal/projecttest0");
@@ -287,7 +287,7 @@ void TestCompletion::methodCall()
         NormalDeclarationCompletionItem* item2 = dynamic_cast<NormalDeclarationCompletionItem*>(item.data());
 
         QString ret;
-        createArgumentList(*item2, ret, 0);
+        createArgumentList(*item2, ret, nullptr);
         QCOMPARE(ret, QString("(A $a, null $b = null)"));
     }
     {
@@ -547,7 +547,7 @@ void TestCompletion::nameClassMember()
 
     PhpCompletionTester tester(top, QStringLiteral("$b->"));
 
-    TestCodeCompletionModel *model = new TestCodeCompletionModel;
+    auto *model = new TestCodeCompletionModel;
     model->foundDeclarations(tester.items, tester.completionContext.data());
 
     QCOMPARE(tester.completionContext->memberAccessOperation(), CodeCompletionContext::MemberAccess);
@@ -1207,7 +1207,7 @@ void TestCompletion::ctorCall()
         NormalDeclarationCompletionItem* item2 = dynamic_cast<NormalDeclarationCompletionItem*>(item.data());
 
         QString ret;
-        createArgumentList(*item2, ret, 0);
+        createArgumentList(*item2, ret, nullptr);
         QCOMPARE(ret, QString("(string $bar)"));
     }
     {
@@ -1222,7 +1222,7 @@ void TestCompletion::ctorCall()
         NormalDeclarationCompletionItem* item2 = dynamic_cast<NormalDeclarationCompletionItem*>(item.data());
 
         QString ret;
-        createArgumentList(*item2, ret, 0);
+        createArgumentList(*item2, ret, nullptr);
         QCOMPARE(ret, QString("(bool $asdf)"));
     }
 }
@@ -1262,7 +1262,7 @@ void TestCompletion::funcCallInConditional()
         NormalDeclarationCompletionItem* item2 = dynamic_cast<NormalDeclarationCompletionItem*>(item.data());
 
         QString ret;
-        createArgumentList(*item2, ret, 0);
+        createArgumentList(*item2, ret, nullptr);
         QCOMPARE(ret, QString("(mixed $a, int $b = 1)"));
     }
 }
