@@ -27,9 +27,10 @@
 #include <language/duchain/declaration.h>
 #include <language/duchain/types/integraltype.h>
 #include "../declarations/classdeclaration.h"
+#include "../types/indexedcontainer.h"
 #include "../types/integraltypeextended.h"
 #include "../types/structuretype.h"
-#include "../duchaindebug.h"
+#include <duchaindebug.h>
 
 #include "editorintegrator.h"
 #include "parsesession.h"
@@ -263,19 +264,6 @@ AbstractType::Ptr TypeBuilder::getTypeForNode(AstNode* node)
     return type;
 }
 
-FunctionType::Ptr TypeBuilder::openFunctionType(AstNode* node)
-{
-    FunctionType::Ptr functionType = FunctionType::Ptr(new FunctionType());
-
-    openType(functionType);
-
-    functionType->setReturnType(parseDocComment(node, QStringLiteral("return")));
-    m_gotReturnTypeFromDocComment = functionType->returnType();
-    updateCurrentType();
-
-    return functionType;
-}
-
 void TypeBuilder::visitClassDeclarationStatement(ClassDeclarationStatementAst* node)
 {
     // the predeclaration builder should have set up a type already
@@ -308,7 +296,15 @@ void TypeBuilder::visitClassStatement(ClassStatementAst *node)
     if (node->methodName) {
         //method declaration
         m_currentFunctionParams = parseDocCommentParams(node);
-        openFunctionType(node);
+
+        FunctionType::Ptr functionType = FunctionType::Ptr(new FunctionType());
+        openType(functionType);
+
+        AbstractType::Ptr phpdocReturnType = parseDocComment(node, QStringLiteral("return"));
+        functionType->setReturnType(returnType(node->returnType, phpdocReturnType, editor(), currentContext()));
+        m_gotReturnTypeFromDocComment = functionType->returnType();
+        updateCurrentType();
+
         TypeBuilderBase::visitClassStatement(node);
         if (currentType<FunctionType>() && !currentType<FunctionType>()->returnType()) {
             currentType<FunctionType>()->setReturnType(AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
@@ -356,36 +352,12 @@ void TypeBuilder::visitConstantDeclaration(ConstantDeclarationAst* node)
 
 void TypeBuilder::visitParameter(ParameterAst *node)
 {
-    AbstractType::Ptr type;
-    if (node->parameterType) {
-        //don't use openTypeFromName as it uses cursor for findDeclarations
-        DeclarationPointer decl = findDeclarationImport(ClassDeclarationType,
-                                                  identifierForNamespace(node->parameterType, editor()));
-        if (decl) {
-            type = decl->abstractType();
-        }
-    } else if (node->arrayType != -1) {
-        type = AbstractType::Ptr(new IntegralType(IntegralType::TypeArray));
-    } else if (node->defaultValue) {
-        ExpressionVisitor v(editor());
-        node->defaultValue->ducontext = currentContext();
-        v.visitNode(node->defaultValue);
-        type = v.result().type();
-    }
-    if (!type) {
-        if (m_currentFunctionParams.count() > currentType<FunctionType>()->arguments().count()) {
-            type = m_currentFunctionParams.at(currentType<FunctionType>()->arguments().count());
-        } else {
-            type = AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed));
-        }
+    AbstractType::Ptr phpDocTypehint;
+    if (m_currentFunctionParams.count() > currentType<FunctionType>()->arguments().count()) {
+        phpDocTypehint = m_currentFunctionParams.at(currentType<FunctionType>()->arguments().count());
     }
 
-    if ( node->isRef != -1 ) {
-      ReferenceType::Ptr p( new ReferenceType() );
-      p->setBaseType( type );
-
-      type = p.cast<AbstractType>();
-    }
+    AbstractType::Ptr type = parameterType(node, phpDocTypehint, editor(), currentContext());
 
     openAbstractType(type);
     TypeBuilderBase::visitParameter(node);
@@ -403,7 +375,8 @@ void TypeBuilder::visitFunctionDeclarationStatement(FunctionDeclarationStatement
     FunctionType::Ptr type = currentType<FunctionType>();
     Q_ASSERT(type);
 
-    type->setReturnType(parseDocComment(node, QStringLiteral("return")));
+    AbstractType::Ptr phpdocReturnType = parseDocComment(node, QStringLiteral("return"));
+    type->setReturnType(returnType(node->returnType, phpdocReturnType, editor(), currentContext()));
     m_gotReturnTypeFromDocComment = type->returnType();
 
     updateCurrentType();
@@ -421,7 +394,8 @@ void TypeBuilder::visitClosure(ClosureAst* node)
     FunctionType::Ptr type = FunctionType::Ptr(new FunctionType());
     openType(type);
 
-    type->setReturnType(parseDocComment(node, QStringLiteral("return")));
+    AbstractType::Ptr phpdocReturnType = parseDocComment(node, QStringLiteral("return"));
+    type->setReturnType(returnType(node->returnType, phpdocReturnType, editor(), currentContext()));
     m_gotReturnTypeFromDocComment = type->returnType();
 
     updateCurrentType();
@@ -471,11 +445,18 @@ void TypeBuilder::visitStatement(StatementAst* node)
                 type = rType->baseType();
             }
             if (ft->returnType() && !ft->returnType()->equals(type.data())) {
+                bool existingTypeIsCallable = ft->returnType().cast<IntegralTypeExtended>() &&
+                    ft->returnType().cast<IntegralTypeExtended>()->dataType() == IntegralTypeExtended::TypeCallable;
+                bool newTypeIsCallable = type.cast<IntegralTypeExtended>() &&
+                    type.cast<IntegralTypeExtended>()->dataType() == IntegralTypeExtended::TypeCallable;
                 if (ft->returnType().cast<IntegralType>()
                     && ft->returnType().cast<IntegralType>()->dataType() == IntegralType::TypeMixed)
                 {
                     //don't add TypeMixed to the list, just ignore
                     ft->setReturnType(type);
+                } else if ((existingTypeIsCallable && type.cast<FunctionType>()) || (newTypeIsCallable && ft->returnType().cast<FunctionType>())) {
+                    //If one type is "callable" and the other a real function, the result is just a "callable".
+                    ft->setReturnType(AbstractType::Ptr(new IntegralTypeExtended(IntegralTypeExtended::TypeCallable)));
                 } else {
                     UnsureType::Ptr retT;
                     if (ft->returnType().cast<UnsureType>()) {
@@ -508,7 +489,7 @@ void TypeBuilder::visitStatement(StatementAst* node)
         }
     }
 
-    AstNode *foreachNode = 0;
+    AstNode *foreachNode = nullptr;
     if (node->foreachVar) {
         foreachNode = node->foreachVar;
     } else if (node->foreachExpr) {
@@ -528,7 +509,7 @@ void TypeBuilder::visitStatement(StatementAst* node)
                 ///FIXME: this is just a hack for https://bugs.kde.org/show_bug.cgi?id=269369
                 ///       a proper fix needs full fledged two-pass, i.e. get rid of PreDeclarationBuilder
                 // 0 == global lookup and the delcaration is found again...
-                classDec = dynamic_cast<ClassDeclaration*>(type->declaration(0));
+                classDec = dynamic_cast<ClassDeclaration*>(type->declaration(nullptr));
             }
             if (classDec) {
                 /// Qualified identifier for 'iterator'
@@ -540,12 +521,15 @@ void TypeBuilder::visitStatement(StatementAst* node)
                 if (classDec->isPublicBaseClass(iteratorDecl, currentContext()->topContext())) {
                     /// Qualified identifier for 'current'
                     static const QualifiedIdentifier currentQId(QStringLiteral("current"));
-                    foreach (Declaration *d, classDec->internalContext()->findDeclarations(currentQId)) {
-                        if (!dynamic_cast<ClassMethodDeclaration*>(d)) continue;
-                        Q_ASSERT(d->type<FunctionType>());
-                        injectType(d->type<FunctionType>()->returnType());
-                        foundType = true;
-                        // qCDebug(DUCHAIN) << "that's it: " << d->type<FunctionType>()->returnType()->toString();
+                    auto classContext = classDec->internalContext();
+                    if (classContext) {
+                        foreach (Declaration *d, classContext->findDeclarations(currentQId)) {
+                            if (!dynamic_cast<ClassMethodDeclaration*>(d)) continue;
+                            Q_ASSERT(d->type<FunctionType>());
+                            injectType(d->type<FunctionType>()->returnType());
+                            foundType = true;
+                            // qCDebug(DUCHAIN) << "that's it: " << d->type<FunctionType>()->returnType()->toString();
+                        }
                     }
                 }
             }
